@@ -143,7 +143,7 @@ I left use clause untouched as I didn't find a suitable translation for it.
 #ifndef TRANSLATOR_SE_H
 #define TRANSLATOR_SE_H
 
-class TranslatorSwedish : public Translator
+class TranslatorSwedish : public TranslatorAdapter_1_8_19
 {
   public:
 
@@ -169,6 +169,11 @@ class TranslatorSwedish : public Translator
       return "\\usepackage[swedish]{babel}\n";
     }
 
+    virtual QCString trISOLang()
+    {
+      return "sv";
+    }
+
     // --- Language translation methods -------------------
 
     /*! used in the compound documentation before a list of related functions. */
@@ -1975,7 +1980,7 @@ class TranslatorSwedish : public Translator
      */
     virtual QCString trPanelSynchronisationTooltip(bool enable)
     {
-      QCString opt = enable ? "aktivera" : "inaktivera"; 
+      QCString opt = enable ? "aktivera" : "inaktivera";
       return "klicka för att "+opt+" panelsynkronisering";
     }
 
@@ -2020,14 +2025,6 @@ class TranslatorSwedish : public Translator
       return "Metoddokumentation";
     }
 
-    /*! Used as the title of the design overview picture created for the
-     *  VHDL output.
-     */
-    virtual QCString trDesignOverview()
-    {
-      return "Designöversikt";
-    }
-
 //////////////////////////////////////////////////////////////////////////
 // new since 1.8.4
 //////////////////////////////////////////////////////////////////////////
@@ -2097,9 +2094,9 @@ class TranslatorSwedish : public Translator
     /** VHDL design unit members */
     virtual QCString trDesignUnitMembers()
     { return "Designenhetsmedlemmar"; }
-    /** VHDL design unit list description 
-     * Orginal: Here is a list of all design unit members with links to 
-     *          the Entities they belong to: 
+    /** VHDL design unit list description
+     * Orginal: Here is a list of all design unit members with links to
+     *          the Entities they belong to:
      */
     virtual QCString trDesignUnitListDescription()
     {
@@ -2336,6 +2333,7 @@ class TranslatorSwedish : public Translator
     {
         return "Datamedlemsdokumentation";
     }
-};
 
+
+};
 #endif
